import contextlib
import gzip
import os


@contextlib.contextmanager
def datafile(name, ungzip=False):
    with open(
        os.path.join(os.path.dirname(__file__), "files", name), mode="rb"
    ) as f:

        if ungzip:
            with gzip.GzipFile(fileobj=f) as gz:
                yield gz
        else:
            yield f


def eq_(a, b):
    assert a == b, "%r != %r" % (a, b)


def file_as_bytes(name, ungzip=False):
    with datafile(name, ungzip=ungzip) as file_:
        return file_.read()


def file_as_string(name, ungzip=False):
    with open(
        os.path.join(os.path.dirname(__file__), "files", name), mode="r"
    ) as f:
        return f.read()



from __future__ import absolute_import

from .parse import dump_nbt  # noqa
from .parse import parse_nbt  # noqa
from .yamlgen import dump_yaml  # noqa
from .yamlgen import parse_yaml  # noqa

__version__ = "0.4.1"



import sys

py3k = sys.version_info > (3,)

if py3k:
    long_ = int
else:
    long_ = long  # noqa F821


if py3k:
    from io import StringIO
    from io import BytesIO
else:
    from StringIO import StringIO  # noqa

    BytesIO = StringIO

if py3k:

    def u(s):
        return s

    def ue(s):
        return s

    def utf8unicode(s):
        assert isinstance(s, str)
        return s

    def utf8str(s):
        assert isinstance(s, str)
        return s


else:

    def u(s):
        return unicode(s, "utf-8")  # noqa

    def ue(s):
        return unicode(s, "unicode_escape")  # noqa

    def utf8unicode(s):
        return s.decode("utf-8")

    def utf8str(s):
        return s.encode("utf-8")


if py3k:
    range = range  # noqa A001
else:
    range = xrange  # noqa F821



import gzip

import pytest

from nbt2yaml import dump_nbt
from nbt2yaml import parse_nbt
from nbt2yaml.compat import BytesIO
from . import datafile
from . import eq_


@pytest.mark.parametrize(
    "name",
    [
        ("test",),
        ("list",),
        ("bigtest",),
        ("spawner",),
        ("intarraytest",),
        ("longarraytest",),
        ("chunk",),
        ("empty_compound",),
    ],
)
def test_dump_nbt(name):
    nbt_name = "%s.nbt" % name
    with datafile(nbt_name) as df:
        unzipped_data = gzip.GzipFile(fileobj=df).read()

    with datafile(nbt_name) as df:
        parsed = parse_nbt(df)

    out = BytesIO()
    dump_nbt(parsed, out)
    out = gzip.GzipFile(fileobj=BytesIO(out.getvalue())).read()
    eq_(unzipped_data, out)



from __future__ import absolute_import
from __future__ import print_function

import argparse
import os
import shutil
import sys
import tempfile

import editor

from . import compat
from . import dump_nbt
from . import dump_yaml
from . import parse_nbt
from . import parse_yaml
from .compat import BytesIO
from .compat import StringIO


def nbtedit():
    parser = argparse.ArgumentParser(
        description="Edit an nbt file in-place in yaml format."
    )
    parser.add_argument("filename", type=str, help="filename")
    parser.add_argument(
        "-n", "--no-gzip", action="store_true", help="Don't use gzip"
    )

    options = parser.parse_args()

    struct = parse_nbt(
        open(options.filename, "rb"), gzipped=not options.no_gzip
    )

    edit_file = tempfile.NamedTemporaryFile(
        delete=False, suffix=".yml", mode="w"
    )
    edit_file.write(dump_yaml(struct))
    edit_file.close()
    edit_filename = edit_file.name
    editor.edit(edit_filename)

    edit_file = open(edit_filename, "rb")
    new_struct = parse_yaml(edit_file)
    edit_file.close()
    os.remove(edit_filename)

    if new_struct == struct:
        sys.stderr.write("No changes made\n")
        sys.exit()

    save_counter = 0
    while True:
        if save_counter == 0:
            savefile = os.extsep.join([options.filename, "saved"])
        else:
            savefile = os.extsep.join(
                [options.filename, str(save_counter), "saved"]
            )
        if not os.path.exists(savefile):
            break
        save_counter += 1

    shutil.move(options.filename, savefile)
    sys.stderr.write("Saving old file as %s\n" % savefile)
    write_file = open(options.filename, "wb")
    sys.stderr.write("Writing %s\n" % options.filename)
    dump_nbt(new_struct, write_file, gzipped=not options.no_gzip)
    write_file.close()


def nbt2yaml():
    parser = argparse.ArgumentParser(
        description="Dump an nbt file or stream to yaml."
    )
    parser.add_argument(
        "filename",
        type=str,
        help="Filename.  Specify as '-' to read from stdin.",
    )
    parser.add_argument(
        "-n", "--no-gzip", action="store_true", help="Don't use gzip"
    )
    options = parser.parse_args()
    if options.filename == "-":
        if compat.py3k:
            in_ = sys.stdin.buffer
        else:
            in_ = sys.stdin
        input_ = BytesIO(in_.read())
    else:
        input_ = open(options.filename, "rb")

    try:
        struct = parse_nbt(input_, gzipped=not options.no_gzip)
        print(dump_yaml(struct))
    finally:
        input_.close()


def yaml2nbt():
    parser = argparse.ArgumentParser(
        description="Dump a yaml file or stream to nbt."
    )
    parser.add_argument(
        "filename",
        type=str,
        help="Filename.  Specify as '-' to read from stdin.",
    )
    parser.add_argument(
        "-n", "--no-gzip", action="store_true", help="Don't use gzip"
    )
    options = parser.parse_args()
    if options.filename == "-":
        input_ = StringIO(sys.stdin.read())
    else:
        input_ = open(options.filename, "rb")

    try:
        struct = parse_yaml(input_)

        if compat.py3k:
            out = sys.stdout.buffer
        else:
            out = sys.stdout
        dump_nbt(struct, out, gzipped=not options.no_gzip)
    finally:
        input_.close()



import pytest

from nbt2yaml import compat
from nbt2yaml import dump_yaml
from nbt2yaml import parse_nbt
from . import datafile
from . import eq_
from . import file_as_string


@pytest.mark.parametrize(
    "name",
    [
        ("test",),
        ("list",),
        ("bigtest",),
        ("spawner",),
        ("intarraytest",),
        ("longarraytest",),
        ("chunk",),
        ("empty_compound",),
    ],
)
def test_to_yaml(name):
    nbt_name = "%s.nbt" % name
    yml_name = "%s.yml" % name

    if not compat.py3k and yml_name == "bigtest.yml":
        yml_name = "bigtest.py2k.yml"

    with datafile(nbt_name) as file_:
        data = parse_nbt(file_)
    eq_(dump_yaml(data), file_as_string(yml_name))



import struct

from nbt2yaml import compat
from nbt2yaml import parse
from nbt2yaml import parse_nbt
from . import datafile
from . import eq_


def test_basic():
    assert_ = (
        parse.TAG_Compound,
        "hello world",
        [(parse.TAG_String, "name", "Bananrama")],
    )
    with datafile("test.nbt") as file_:
        eq_(parse_nbt(file_), assert_)


def test_lists():
    assert_ = (
        parse.TAG_Compound,
        "",
        [
            (
                parse.TAG_Compound,
                "Data",
                [
                    (parse.TAG_Byte, "thundering", 0),
                    (parse.TAG_Long, "LastPlayed", 1315921966180),
                    (
                        parse.TAG_Compound,
                        "Player",
                        [
                            (
                                parse.TAG_List,
                                "Motion",
                                (
                                    parse.TAG_Double,
                                    [
                                        9.166176096485612e-17,
                                        -0.0784000015258789,
                                        -2.063101401779548e-16,
                                    ],
                                ),
                            ),
                            (parse.TAG_Short, "Health", 20),
                            (
                                parse.TAG_List,
                                "Inventory",
                                (parse.TAG_Byte, []),
                            ),
                        ],
                    ),
                ],
            )
        ],
    )
    with datafile("list.nbt") as file_:
        eq_(parse_nbt(file_), assert_)


def test_large():
    assert_ = (
        parse.TAG_Compound,
        "Level",
        [
            (parse.TAG_Long, "longTest", 9223372036854775807),
            (parse.TAG_Short, "shortTest", 32767),
            (
                parse.TAG_String,
                "stringTest",
                compat.ue("HELLO WORLD THIS IS A TEST STRING \xc5\xc4\xd6!"),
            ),
            (parse.TAG_Float, "floatTest", 0.4982314705848694),
            (parse.TAG_Int, "intTest", 2147483647),
            (
                parse.TAG_Compound,
                "nested compound test",
                [
                    (
                        parse.TAG_Compound,
                        "ham",
                        [
                            (parse.TAG_String, "name", "Hampus"),
                            (parse.TAG_Float, "value", 0.75),
                        ],
                    ),
                    (
                        parse.TAG_Compound,
                        "egg",
                        [
                            (parse.TAG_String, "name", "Eggbert"),
                            (parse.TAG_Float, "value", 0.5),
                        ],
                    ),
                ],
            ),
            (
                parse.TAG_List,
                "listTest (long)",
                (parse.TAG_Long, [11, 12, 13, 14, 15]),
            ),
            (
                parse.TAG_List,
                "listTest (compound)",
                (
                    parse.TAG_Compound,
                    [
                        [
                            (parse.TAG_String, "name", "Compound tag #0"),
                            (parse.TAG_Long, "created-on", 1264099775885),
                        ],
                        [
                            (parse.TAG_String, "name", "Compound tag #1"),
                            (parse.TAG_Long, "created-on", 1264099775885),
                        ],
                    ],
                ),
            ),
            (parse.TAG_Byte, "byteTest", 127),
            (
                parse.TAG_Byte_Array,
                "byteArrayTest (the first 1000 values of "
                "(n*n*255+n*7)%100, starting with n=0 "
                "(0, 62, 34, 16, 8, ...))",
                b"".join(
                    [
                        struct.pack(">b", (n * n * 255 + n * 7) % 100)
                        for n in compat.range(0, 1000)
                    ]
                ),
            ),
            (parse.TAG_Double, "doubleTest", 0.4931287132182315),
        ],
    )

    with datafile("bigtest.nbt") as file_:
        eq_(parse_nbt(file_), assert_)


def test_schematic():
    assert_ = (
        parse.TAG_Compound,
        "Schematic",
        [
            (parse.TAG_Byte_Array, "Blocks", b"\x34"),
            (parse.TAG_Short, "Width", 1),
            (parse.TAG_Short, "Height", 1),
            (parse.TAG_List, "Entities", (parse.TAG_Byte, [])),
            (parse.TAG_Short, "Length", 1),
            (parse.TAG_String, "Materials", "Alpha"),
            (
                parse.TAG_List,
                "TileEntities",
                (
                    parse.TAG_Compound,
                    [
                        [
                            (parse.TAG_Short, "Delay", 120),
                            (parse.TAG_Int, "y", 0),
                            (parse.TAG_Int, "x", 0),
                            (parse.TAG_String, "EntityId", "Skeleton"),
                            (parse.TAG_Int, "z", 0),
                            (parse.TAG_String, "id", "MobSpawner"),
                        ]
                    ],
                ),
            ),
            (parse.TAG_Byte_Array, "Data", b"\x00"),
        ],
    )
    with datafile("spawner.nbt") as file_:
        eq_(parse_nbt(file_), assert_)


def test_int_array():
    assert_ = (
        parse.TAG_Compound,
        "Test root compound",
        [
            (
                parse.TAG_Int_Array,
                "Test integer array",
                [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16],
            )
        ],
    )
    with datafile("intarraytest.nbt") as file_:
        eq_(parse_nbt(file_), assert_)



import pytest

from nbt2yaml import compat
from nbt2yaml import dump_nbt
from nbt2yaml import parse_nbt
from nbt2yaml import parse_yaml
from nbt2yaml.compat import BytesIO
from . import datafile
from . import eq_


@pytest.mark.parametrize(
    "name",
    [
        ("test",),
        ("list",),
        ("bigtest",),
        ("spawner",),
        ("intarraytest",),
        ("longarraytest",),
        ("chunk",),
        ("empty_compound",),
    ],
)
def test_from_yaml(name):
    nbt_name = "%s.nbt" % name
    yml_name = "%s.yml" % name

    if not compat.py3k and yml_name == "bigtest.yml":
        yml_name = "bigtest.py2k.yml"

    with datafile(yml_name) as file_:
        data = parse_yaml(file_)

    with datafile(nbt_name) as file_:
        eq_(data, parse_nbt(file_))

    out = BytesIO()
    dump_nbt(data, out, gzipped=False)

    with datafile(nbt_name, ungzip=True) as file_:
        eq_(out.getvalue(), file_.read())



from __future__ import absolute_import

from collections import namedtuple
import gzip
import re
import struct

from . import compat


class Tag(object):
    _tags = {}
    _tuple = namedtuple("Tag", ["type", "name", "data"])

    def __init__(self, name, id_):
        self.name = name
        self.id = id_
        Tag._tags[id_] = self

    @classmethod
    def from_stream(cls, stream):
        return cls._tags[struct.unpack(">b", stream.read(1))[0]]

    def to_stream(self, stream):
        stream.write(struct.pack(">b", self.id))

    def parse(self, stream):
        name = TAG_String._parse_impl(stream)
        data = self._parse_impl(stream)
        return Tag._tuple(self, name, data)

    def dump(self, element, stream):
        type_, name, data = element
        type_.to_stream(stream)
        TAG_String._dump_impl(name, stream)
        self._dump_impl(data, stream)

    def _parse_length(self, length_type, stream):
        return struct.unpack(
            ">" + length_type.format, stream.read(length_type.size)
        )[0]

    def _dump_length(self, length_type, length, stream):
        stream.write(struct.pack(">" + length_type.format, length))

    def _parse_impl(self, stream):
        raise NotImplementedError()

    def _dump_impl(self, data, stream):
        raise NotImplementedError()

    def __repr__(self):
        return "TAG_%s" % re.sub(
            r"^(\w)|_(\w)", lambda m: m.group(0).upper(), self.name
        )


class FixedTag(Tag):
    def __init__(self, name, id_, size, format_):
        Tag.__init__(self, name, id_)
        self.size = size
        self.format = format_

    def _parse_impl(self, stream):
        return struct.unpack(">" + self.format, stream.read(self.size))[0]

    def _dump_impl(self, data, stream):
        stream.write(struct.pack(">" + self.format, data))


class EndTag(Tag):
    pass


class VariableTag(Tag):
    def __init__(self, name, id_, length_tag, encoding=None):
        Tag.__init__(self, name, id_)
        self.length_tag = length_tag
        self.encoding = encoding

    def _parse_impl(self, stream):
        length = self._parse_length(self.length_tag, stream)
        data = stream.read(length)
        if self.encoding:
            data = data.decode(self.encoding)
        return data

    def _dump_impl(self, data, stream):
        if self.encoding:
            data = data.encode(self.encoding)
        self._dump_length(self.length_tag, len(data), stream)
        stream.write(data)


class ListTag(Tag):
    def _parse_impl(self, stream):
        element_type = Tag.from_stream(stream)
        length = self._parse_length(TAG_Int, stream)

        return (
            element_type,
            [element_type._parse_impl(stream) for i in compat.range(length)],
        )

    def _dump_impl(self, data, stream):
        element_type, data = data
        element_type.to_stream(stream)
        self._dump_length(TAG_Int, len(data), stream)
        for elem in data:
            element_type._dump_impl(elem, stream)


class CompoundTag(Tag):
    def _parse_impl(self, stream):
        data = []
        while True:
            c_type_ = Tag.from_stream(stream)
            if c_type_ is TAG_End:
                break
            data.append(c_type_.parse(stream))
        return data

    def _dump_impl(self, data, stream):
        for elem in data:
            elem[0].dump(elem, stream)
        TAG_End.to_stream(stream)


class IntArrayTag(Tag):
    def _parse_impl(self, stream):
        element_type = TAG_Int
        length = self._parse_length(TAG_Int, stream)

        return [element_type._parse_impl(stream) for i in compat.range(length)]

    def _dump_impl(self, data, stream):
        self._dump_length(TAG_Int, len(data), stream)
        for elem in data:
            TAG_Int._dump_impl(elem, stream)


class LongArrayTag(Tag):
    def _parse_impl(self, stream):
        element_type = TAG_Long
        length = self._parse_length(TAG_Long, stream)

        return [element_type._parse_impl(stream) for i in compat.range(length)]

    def _dump_impl(self, data, stream):
        self._dump_length(TAG_Long, len(data), stream)
        for elem in data:
            TAG_Long._dump_impl(elem, stream)


TAG_End = EndTag("end", 0)
TAG_Byte = FixedTag("byte", 1, 1, "b")
TAG_Short = FixedTag("short", 2, 2, "h")
TAG_Int = FixedTag("int", 3, 4, "i")
TAG_Long = FixedTag("long", 4, 8, "q")
TAG_Float = FixedTag("float", 5, 4, "f")
TAG_Double = FixedTag("double", 6, 8, "d")
TAG_Byte_Array = VariableTag("byte_array", 7, TAG_Int)
TAG_String = VariableTag("string", 8, TAG_Short, encoding="utf-8")
TAG_List = ListTag("list", 9)
TAG_Compound = CompoundTag("compound", 10)
TAG_Int_Array = IntArrayTag("int_array", 11)
TAG_Long_Array = LongArrayTag("long_array", 12)


def parse_nbt(stream, gzipped=True):
    """Parse an .nbt file from the given stream.

    :param stream: a file-like stream configured for read().
    :param gzipped: if True, assume the stream is in gzip format first.
    :return: a Python structure representing the .nbt file.

    """
    if gzipped:
        stream = gzip.GzipFile(fileobj=stream)
    type_ = Tag.from_stream(stream)
    return type_.parse(stream)


def dump_nbt(nbt, stream, gzipped=True):
    """Dump an .nbt structure generated by :func:`.parse_nbt`
    to the given stream.

    :param nbt: the structure returned by a :func:`.parse_nbt` call.
    :param stream: a file-like stream configured for write().
    :param gzipped: if True, pass the stream through gzip for final output.

    """
    if gzipped:
        stream = gzip.GzipFile(mode="wb", fileobj=stream)
    type_ = nbt[0]
    type_.dump(nbt, stream)



import os
import re

from setuptools import setup

v = open(os.path.join(os.path.dirname(__file__), "nbt2yaml", "__init__.py"))
VERSION = (
    re.compile(r""".*__version__ = ["'](.*?)["']""", re.S)
    .match(v.read())
    .group(1)
)
v.close()

setup(version=VERSION)



from __future__ import absolute_import

import yaml

from nbt2yaml import parse
from . import compat

_explicit_types = (
    parse.TAG_Short,
    parse.TAG_Long,
    parse.TAG_Double,
    parse.TAG_Byte,
    parse.TAG_Byte_Array,
)
_all_types = parse.Tag._tags.values()

_canned_types = {
    str: parse.TAG_String,
    float: parse.TAG_Float,
    int: parse.TAG_Int,
}

_types_to_python = {
    parse.TAG_Long: compat.long_,
    parse.TAG_Int: int,
    parse.TAG_Short: int,
    parse.TAG_Byte: int,
    parse.TAG_Float: float,
    parse.TAG_Double: float,
}


class ForceType(object):
    """Represent a data value with an explicit type.

    This is used to output 'short', 'long', 'double', 'byte'
    explicitly, so that we can differentiate on the
    yaml parsing side what specific NBT form to use.

    """

    def __init__(self, type_, value):
        self.type = type_
        self.value = value


class ForceListOfType(ForceType):
    pass


class ForceIntArray(ForceType):
    pass


class ForceLongArray(ForceType):
    pass


if compat.py3k:

    def _hex_dump(binary):
        hexa = []
        for i in binary:
            hexa.append("{0:02X}".format(i))
        return " ".join(hexa)

    def _hex_undump(hexa):
        binary = bytearray()
        hexa = hexa.split(" ")
        for i in hexa:
            binary.append(int(i, 16))
        return bytes(binary)


else:

    def _hex_dump(binary):
        hexa = []
        for i in binary:
            hexa.append("{0:02X}".format(ord(i)))
        return " ".join(hexa)

    def _hex_undump(hexa):
        binary = ""
        hexa = hexa.split(" ")
        for i in hexa:
            current = int(i, 16)
            binary += chr(current)
        return binary


def _type_representer(dumper, struct):
    if struct.type is parse.TAG_Byte_Array:
        representation = _hex_dump(struct.value)
    else:
        representation = repr(struct.value)
    return dumper.represent_scalar(
        "!%s" % struct.type.name, representation, style='""'
    )


def _collection_representer(dumper, struct):
    return dumper.represent_sequence(
        "!list_%s" % struct.type.name, struct.value
    )


def _int_array_representer(dumper, struct):
    return dumper.represent_sequence("!int_array", struct.value)


def _long_array_representer(dumper, struct):
    return dumper.represent_sequence("!long_array", struct.value)


yaml.add_representer(ForceType, _type_representer)
yaml.add_representer(ForceListOfType, _collection_representer)
yaml.add_representer(ForceIntArray, _int_array_representer)
yaml.add_representer(ForceLongArray, _long_array_representer)


def _type_constructor(type_):
    def _constructor(loader, node):
        value = loader.construct_scalar(node)
        return ForceType(type_, value)

    return _constructor


def _list_constructor(type_):
    def _constructor(loader, node):
        value = loader.construct_sequence(node)
        return ForceListOfType(type_, value)

    return _constructor


def _int_array_constructor(loader, node):
    value = loader.construct_sequence(node)
    return ForceIntArray(parse.TAG_Int_Array, value)


def _byte_array_constructor(loader, node):
    value = loader.construct_scalar(node)
    return ForceType(parse.TAG_Byte_Array, _hex_undump(value))


def _long_array_constructor(loader, node):
    value = loader.construct_sequence(node)
    return ForceLongArray(parse.TAG_Long_Array, value)


for type_ in _explicit_types:
    if type_ is not parse.TAG_Byte_Array:
        yaml.add_constructor("!%s" % type_.name, _type_constructor(type_))

for type_ in _all_types:
    yaml.add_constructor("!list_%s" % type_.name, _list_constructor(type_))

yaml.add_constructor("!int_array", _int_array_constructor)
yaml.add_constructor("!byte_array", _byte_array_constructor)
yaml.add_constructor("!long_array", _long_array_constructor)


def _yaml_serialize(struct):
    tag, name, data = struct.type, struct.name, struct.data
    name = compat.utf8str(name)
    value = _value_as_yaml(tag, data)
    return {name: value}


def _yaml_deserialize(struct):
    name, data = list(struct.items())[0]
    type_ = _type_from_yaml(data)
    return parse.Tag._tuple(
        type_, compat.utf8unicode(name), _yaml_as_value(type_, data)
    )


def _value_as_yaml(type_, value):
    if type_ is parse.TAG_Compound:
        return [_yaml_serialize(s) for s in value]
    elif type_ is parse.TAG_String:
        return compat.utf8str(value)
    elif type_ in _explicit_types:
        return ForceType(type_, value)
    elif type_ is parse.TAG_List:
        element_type, data = value
        return ForceListOfType(
            element_type, [_value_as_yaml(element_type, d) for d in data]
        )
    elif type_ is parse.TAG_Int_Array:
        return ForceIntArray(parse.TAG_Int, value)
    elif type_ is parse.TAG_Long_Array:
        return ForceLongArray(parse.TAG_Long, value)
    else:
        return value


def _yaml_as_value(type_, value):
    if type_ is parse.TAG_Compound:
        return [_yaml_deserialize(s) for s in value]
    elif type_ is parse.TAG_String:
        return compat.utf8unicode(value)
    elif type_ in _explicit_types:
        if type_ in _types_to_python:
            return _types_to_python[type_](value.value)
        else:
            return value.value
    elif type_ is parse.TAG_List:
        ltype = value.type
        return (ltype, [_yaml_as_value(ltype, s) for s in value.value])
    elif type_ is parse.TAG_Int_Array:
        return value.value
    elif type_ is parse.TAG_Long_Array:
        return value.value
    else:
        return value


def _type_from_yaml(data):
    if isinstance(data, ForceListOfType):
        return parse.TAG_List
    elif isinstance(data, ForceIntArray):
        return parse.TAG_Int_Array
    elif isinstance(data, ForceLongArray):
        return parse.TAG_Long_Array
    elif isinstance(data, list):
        return parse.TAG_Compound
    elif isinstance(data, ForceType):
        return data.type
    elif type(data) in _canned_types:
        return _canned_types[type(data)]
    else:
        raise ValueError("Can't determine type for element: %r" % data)


def dump_yaml(struct, canonical=False, default_flow_style=False):
    """Dump a yaml string given an nbt structure parsed by nbt2yaml."""

    return yaml.dump(
        _yaml_serialize(struct),
        default_flow_style=default_flow_style,
        canonical=canonical,
    )


def parse_yaml(stream):
    """Parse a yaml stream into an nbt structure readable by nbt2yaml."""

    return _yaml_deserialize(yaml.load(stream, Loader=yaml.FullLoader))
